/*
 * Copyright 2015-2016 Red Hat, Inc, and individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.hal.client.configuration.subsystem.microprofile;

import org.jboss.hal.ballroom.VerticalNavigation;
import org.jboss.hal.ballroom.form.Form;
import org.jboss.hal.ballroom.table.Table;
import org.jboss.hal.core.mbui.MbuiContext;
import org.jboss.hal.core.mbui.MbuiViewImpl;
import org.jboss.hal.dmr.ModelNode;
import org.jboss.hal.dmr.NamedNode;
import org.jboss.hal.spi.MbuiElement;
import org.jboss.hal.spi.MbuiView;

import static org.jboss.hal.dmr.ModelNodeHelper.asNamedNodes;
import static org.jboss.hal.dmr.ModelNodeHelper.failSafePropertyList;

@MbuiView
@SuppressWarnings({"WeakerAccess", "unused"})
public abstract class MicroProfileConfigView extends MbuiViewImpl<MicroProfileConfigPresenter>
        implements MicroProfileConfigPresenter.MyView {

    public static MicroProfileConfigView create(MbuiContext mbuiContext) {
        return new Mbui_MicroProfileConfigView(mbuiContext);
    }

    @MbuiElement("microprofile-config-vertical-navigation") VerticalNavigation navigation;
    @MbuiElement("microprofile-config-source-table") Table<NamedNode> configSourceTable;
    @MbuiElement("microprofile-config-source-form") Form<NamedNode> configSourceForm;
    @MbuiElement("microprofile-config-source-provider-table") Table<NamedNode> configSourceProviderTable;
    @MbuiElement("microprofile-config-source-provider-form") Form<NamedNode> configSourceProviderForm;

    MicroProfileConfigView(MbuiContext mbuiContext) {
        super(mbuiContext);
    }

    @Override
    public void update(ModelNode modelNode) {
        configSourceForm.clear();
        configSourceTable.update(asNamedNodes(failSafePropertyList(modelNode, "config-source")));
        configSourceProviderForm.clear();
        configSourceProviderTable.update(asNamedNodes(failSafePropertyList(modelNode, "config-source-provider")));
    }
}
